--
-- LSUpgrade
--
-- @author  ANDR55
-- Version 0.1
-- @date 12.10.2014

LSUpgrade = {};


local esl_directory = g_currentModDirectory;

source(g_currentModDirectory.."LSUpgradeEvent.lua");
source(g_currentModDirectory.."LSUpgradeRefreshEvent.lua");

function LSUpgrade.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;


function LSUpgrade:load(xmlFile)
	
	self.updateUpdgrade = LSUpgrade.updateUpdgrade;
	
	self.upgrades = {};
	

	if self.capacity ~= nil then
	
		local upgrade = {name = "capacity", level = 0, value_orig = self.capacity, increase = 10, maxlevel = 3, costpercent = 10, costpercentincrease = 5, unit = "l"}
	
		table.insert(self.upgrades, upgrade);

		self.capacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.maxlevel, 0)));
	end;
	
	if self.grainTankCapacity ~= nil then
	
		local upgrade = {name = "grainTankCapacity", level = 0, value_orig = self.grainTankCapacity, increase = 10, maxlevel = 3, costpercent = 10, costpercentincrease = 5, unit = "l"}
	
		table.insert(self.upgrades, upgrade);
	
		self.grainTankCapacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.maxlevel, 0)));
	end;

	if self.fuelCapacity ~= nil then
	
		local upgrade = {name = "fuelCapacity", level = 0, value_orig = self.fuelCapacity, increase = 5, maxlevel = 10, costpercent = 1, costpercentincrease = 1, unit = "l"}
	
		table.insert(self.upgrades, upgrade);
	
		self.fuelCapacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.maxlevel, 0)));
	end;

	if self.grainTankUnloadingCapacity ~= nil then
	
		local upgrade = {name = "grainTankUnloadingCapacity", level = 0, value_orig = self.grainTankUnloadingCapacity, increase = 10, maxlevel = 3, costpercent = 5, costpercentincrease = 5, unit = "l/s"}
	
		table.insert(self.upgrades, upgrade);
	end;

	if self.threshingScale ~= nil then
	
		local upgrade = {name = "threshingScale", level = 0, value_orig = self.threshingScale, increase = 5, maxlevel = 5, costpercent = 10, costpercentincrease = 5, displaymultiplier = 100, unit = "%"}
	
		table.insert(self.upgrades, upgrade);
	end;

	if self.pricePerMS ~= nil then
	
		local difficultyMultiplier = Utils.lerp(0.6, 1, (g_currentMission.missionStats.difficulty - 1) / 2)
	
		local upgrade = {name = "pricePerMS", level = 0, value_orig = self.pricePerMS, increase = -5, maxlevel = 5, costpercent = 10, costpercentincrease = 5, displaymultiplier = 60 * 1000 * difficultyMultiplier, unit = "Eur/Std"}
	
		table.insert(self.upgrades, upgrade);
	end;

	if self.fuelUsage ~= nil then
	
		local upgrade = {name = "fuelUsage", level = 0, value_orig = self.fuelUsage, increase = -5, maxlevel = 5, costpercent = 10, costpercentincrease = 5, displaymultiplier = 100000, unit = "l/100km"}
	
		table.insert(self.upgrades, upgrade);
	end;

	if self.fillLitersPerSecond ~= nil then
	
		local upgrade = {name = "fillLitersPerSecond", level = 0, value_orig = self.fillLitersPerSecond, increase = 5, maxlevel = 5, costpercent = 10, costpercentincrease = 5, unit = "l/s"}
	
		table.insert(self.upgrades, upgrade);
	end;

	if self.fillScale ~= nil then
	
		local upgrade = {name = "fillScale", level = 0, value_orig = self.fillScale, increase = 5, maxlevel = 5, costpercent = 10, costpercentincrease = 5, displaymultiplier = 100, unit = "%"}
	
		table.insert(self.upgrades, upgrade);
	end;
	
	--allowThreshingDuringRain

end;

function LSUpgrade:delete()
end;

function LSUpgrade:readStream(streamId, connection)
	
end;

function LSUpgrade:readUpdateStream(streamId, timestamp, connection)

end

function LSUpgrade:writeStream(streamId, connection)
	
end;

function LSUpgrade:writeUpdateStream(streamId, connection, dirtyMask)

end

function LSUpgrade:mouseEvent(posX, posY, isDown, isUp, button)

end;

function LSUpgrade:keyEvent(unicode, sym, modifier, isDown)

end;

function LSUpgrade:updateTick(dt)

end;

function LSUpgrade:update(dt)
	
	if self.LSUpgrade == nil then
		self.LSUpgrade = {}
	end
	
	if not self.LSUpgrade.initialize then
		--Broadcast auslsen
		g_client:getServerConnection():sendEvent(LSUpgradeRefreshEvent:new(self));
		self.LSUpgrade.initialize = true;
	end;
	
end;

function LSUpgrade:draw()

end;

function LSUpgrade:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

	for _, upgrade in pairs (self.upgrades) do

		local valueStr = getXMLString(xmlFile, key.."#upgrade_"..upgrade.name);
		
		upgrade.level = Utils.getNoNil(valueStr, 0);
	end;
	
	self:updateUpdgrade(resetVehicles);

	return BaseMission.VEHICLE_LOAD_OK;
end;

function LSUpgrade:getSaveAttributesAndNodes(nodeIdent)

	local attributes = "";

	for _, upgrade in pairs (self.upgrades) do

		if attributes ~= "" then
			attributes = attributes.." ";
		end;
	
		attributes = attributes..'upgrade_'..upgrade.name..'="'..Utils.getNoNil(upgrade.level, "0")..'"';
	end;
	
    return attributes, nil;
end;

function LSUpgrade:updateUpdgrade(resetVehicles)

	for _, upgrade in pairs (self.upgrades) do
	
		if upgrade.name == "capacity" then
		
			self.capacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		
			self:setFillLevel(self.fillLevel, self.currentFillType, true);
		end;
	
	
		if upgrade.name == "grainTankCapacity" then
		
			self.grainTankCapacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		
			self:setGrainTankFillLevel(self.grainTankFillLevel, self.currentGrainTankFruitType);
		end;
		
		
		if upgrade.name == "fuelCapacity" then
		
			self.fuelCapacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		
			self:setFuelFillLevel(self.fuelFillLevel);
		end;

		
		if upgrade.name == "grainTankUnloadingCapacity" then
		
			self.grainTankUnloadingCapacity = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;
		
		
		if upgrade.name == "threshingScale" then
		
			self.threshingScale = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;
		
		
		if upgrade.name == "pricePerMS" then
		
			self.pricePerMS = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;

		
		if upgrade.name == "fuelUsage" then
		
			self.fuelUsage = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;


		if upgrade.name == "fillLitersPerSecond" then
		
			self.fillLitersPerSecond = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;
		
		
		if upgrade.name == "fillScale" then
		
			self.fillScale = upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * Utils.getNoNil(upgrade.level, 0)));
		end;
	end;
	
	if resetVehicles then
		g_client:getServerConnection():sendEvent(LSUpgradeRefreshEvent:new(self));
	end;
end;
